#include <SPI.h>
#include <SD.h>
#include <DS3231.h>

DS3231  rtc(SDA, SCL);            
File myFile;

//CONSTANTs
#define AVRG_VOLT 2.50

//PINs
#define A 5
#define B 6
#define C 7
#define INH 8
#define SS_PIN 10

//MACROSes
#define GET_DATA() rtc.getDateStr(FORMAT_SHORT, FORMAT_LITTLEENDIAN, '-')
#define GET_TIME() rtc.getTimeStr(FORMAT_LONG)

float ADC_data[16] = {0};

unsigned long write_time;
unsigned long f_50hz_time;
unsigned long DelayTime = 1000;

byte UART_Time[6] = {0,0,0,0,0,0};
unsigned char index_time = 0;

void ADC_measure(void);
void WriteDataToSD(void);
void ReadAllFiles(void);
void WriteDataToUART(void);

void get_commands(void);


void setup() {
  pinMode(A, OUTPUT);
  pinMode(B, OUTPUT);
  pinMode(C, OUTPUT);
  
  pinMode(INH, OUTPUT);
  digitalWrite(INH, HIGH);
  
  pinMode(SS_PIN, OUTPUT);
  write_time = millis();

  Serial.begin(115200);
  delay(100);
  Serial.println("Before RTC init");
  rtc.begin();
  Serial.println("After RTC init");
  if(!SD.begin(SS_PIN)) 
  {
    Serial.println("SD card init fail.");
  }
}


void loop() {
  get_commands();
  ADC_measure();
  WriteDataToSD();
}




void Set_CHANNELs(unsigned char number)
{
  digitalWrite(INH, HIGH);
   switch (number)
  {
   case 0:digitalWrite(C,0);
          digitalWrite(B,0);
          digitalWrite(A,0);
          break;
   case 1:digitalWrite(C,0);
          digitalWrite(B,0);
          digitalWrite(A,1);
          break;
   case 2:digitalWrite(C,0);
          digitalWrite(B,1);
          digitalWrite(A,0);
          break;
   case 3:digitalWrite(C,0);
          digitalWrite(B,1);
          digitalWrite(A,1);
          break;
   case 4:digitalWrite(C,1);
          digitalWrite(B,0);
          digitalWrite(A,0);
          break;
   case 5:digitalWrite(C,1);
          digitalWrite(B,0);
          digitalWrite(A,1);
          break;
   case 6:digitalWrite(C,1);
          digitalWrite(B,1);
          digitalWrite(A,0);
          break;
   case 7:digitalWrite(C,1);
          digitalWrite(B,1);
          digitalWrite(A,1);
          break;
   default: break;
  }
  digitalWrite(INH, LOW);
}

void ADC_measure(void)
{
  unsigned char i = 0;
  for(i=0; i<8; i++)
  {
    Set_CHANNELs(i);
    f_50hz_time = millis();
    unsigned long k_rms = 0;
    float Irms_0=0.0f, Irms_1=0.0f;
    while(f_50hz_time+20>=millis())
    {
      float voltage0 = (analogRead(A0) * 5.0)/1024.0; 
      float voltage1 = (analogRead(A1) * 5.0)/1024.0;
      Irms_0 += sq((voltage0 - AVRG_VOLT) * 30);
      Irms_1 += sq((voltage1 - AVRG_VOLT) * 30);
      k_rms++;
    }
    ADC_data[i] = sqrt(Irms_0/k_rms);
    ADC_data[i+8] = sqrt(Irms_1/k_rms);
  }
}

void WriteDataToSD(void)
{
  unsigned char i = 0;
  myFile = SD.open(GET_DATA(), FILE_WRITE);
  if((write_time+DelayTime) < millis())
  {
    if(myFile)
    {
      for(i=0; i<16; i++)
      {
        myFile.print(GET_DATA()); myFile.print(';'); myFile.print(GET_TIME()); myFile.print(';'); myFile.print(i+1); myFile.print(';'); myFile.print(ADC_data[i]); myFile.println(';');
      }
      Serial.println("Success writing data to FILE");
    }
    else
    {
      Serial.println("Error read file from SD card");
    }
    write_time = millis();
  }
  myFile.close();
}

void ReadAllFiles(void)
{
  File dir = SD.open("/");
  dir.rewindDirectory();
  File myFile = dir.openNextFile();
  while(myFile)
  {
    Serial.print("\n\r File name: "); Serial.println(myFile.name());
    while(myFile.available())
    {
      Serial.write(myFile.read());
    }
    myFile.close();
    myFile = dir.openNextFile();
  }
  
  Serial.println("NO MORE FILES");
}

void WriteDataToUART(void)
{
  unsigned char i = 0;
  if( (write_time+DelayTime) < millis())
  {
    Serial.println();
    Serial.print(GET_DATA()); Serial.print(" "); Serial.println(GET_TIME());
    for(i=0; i<16; i++)
    {
      Serial.print("CH "); Serial.print(i); Serial.print(": "); Serial.print(ADC_data[i]); Serial.print(" A");Serial.print("\n\r");
    }
    write_time = millis();
  }
}

void get_commands(void)
{
  char cmd;
  if(Serial.available())
  {
    cmd = Serial.read();
  }
  
  switch(cmd)
  {
    case 't':
    {
      //DD MM YY - Day Month Year, HH MM SS - Hours Minutes Seconds
      Serial.println("Input date and time at this format -> DD MM YY HH MM SS ");
      while(!Serial.available());
      while((Serial.available()))
      {
        UART_Time[index_time] = byte(Serial.parseInt());
        index_time++;
      }
      rtc.setDate(UART_Time[0], UART_Time[1], UART_Time[2]);
      rtc.setTime(UART_Time[3], UART_Time[4], UART_Time[5]);
      index_time = 0;
      Serial.print("Date and time set to "); Serial.print(GET_DATA()); Serial.print(" "); Serial.println(GET_TIME());
      break;
    }
    case 'r':
    {
      ReadAllFiles();
      break;
    }
    default:break;
  }
}
